//*----------------------------------------------------------------------------
//*         ATMEL Microcontroller Software Support  -  ROUSSET  -
//*----------------------------------------------------------------------------
//* The software is delivered "AS IS" without warranty or condition of any
//* kind, either express, implied or statutory. This includes without
//* limitation any warranty or condition with respect to merchantability or
//* fitness for any particular purpose, or against the infringements of
//* intellectual property rights of others.
//*----------------------------------------------------------------------------
//* File Name           : dbgu.c
//* Object              : DBGU routines written in C
//* Creation            : JG   16/Aug/2004
//*----------------------------------------------------------------------------

// Include Standard files
#include "board.h"
#include "lcd.h"
#include <string.h>

// set screen display to nothign displayed...
void Lcd_Display_Init()
{
  unsigned int *pLCDC_MEM;
  pLCDC_MEM = (unsigned int *) (AT91C_SLCDC_MEM);	
	
  // Init all Segments blank
  memset(pLCDC_MEM, 10*2*4, 0);
}

#define COMMON_NUMBER   9<<0   //10 commons
#define SEGMENT_NUMBER  39<<8  //40 segments

// initialize the LCD driver (clock, common, segments...)
void Lcd_Init(void)
{
  AT91PS_SLCDC 	pSLCDC = AT91C_BASE_SLCDC;
  if (pSLCDC -> SLCDC_SR == AT91C_SLCDC_ENA) return;
  pSLCDC -> SLCDC_CR = AT91C_SLCDC_SWRST;
  
  pSLCDC -> SLCDC_MR = COMMON_NUMBER | AT91C_SLCDC_BUFTIME_8_Tsclk |
                       AT91C_SLCDC_BIAS_1_3 | SEGMENT_NUMBER;
  pSLCDC -> SLCDC_FRR = AT91C_SLCDC_PRESC_SCLK_16 | AT91C_SLCDC_DIV_2;
  pSLCDC -> SLCDC_CR = AT91C_SLCDC_LCDEN;

  //while (pSLCDC -> SLCDC_SR != AT91C_SLCDC_ENA); //TODO: put back...
}

// disable the LCD drivers...
void Lcd_Disable()
{
  AT91C_BASE_SLCDC->SLCDC_CR = AT91C_SLCDC_LCDDIS; // BDs
  AT91C_BASE_SUPC->SUPC_MR = (0xA5UL << 24) | (AT91C_BASE_SUPC->SUPC_MR & ~AT91C_SUPC_LCDMODE); // BDs
  while ((AT91C_BASE_SUPC->SUPC_SR & AT91C_SUPC_LCDS) == AT91C_SUPC_LCDS); // BDs
}

void Lcd_SetContrast(int contrast)
{
  AT91C_BASE_SUPC->SUPC_MR = (((AT91C_BASE_SUPC->SUPC_MR)|(0xA5UL << 24))&0xFFFFFFF0)|(contrast&0xf);
}

// sets default values for LCD...
// including power supply....
void LCDMODE_Select(int LCDMODE)
{
  AT91C_BASE_SUPC->SUPC_MR = ((AT91C_BASE_SUPC->SUPC_MR|(0xA5UL << 24))&0xFFFFFFCF)|LCDMODE;

  if (LCDMODE==AT91C_SUPC_LCDMODE_OFF || LCDMODE==AT91C_SUPC_LCDMODE_OFF_AFTER_EOF)
    while( (AT91C_BASE_SUPC->SUPC_SR & AT91C_SUPC_LCDS) ); 
  else
      while( !(AT91C_BASE_SUPC->SUPC_SR & AT91C_SUPC_LCDS) ); 
}
void Lcd_Enable(int contrast)
{
  LCDMODE_Select(AT91C_SUPC_LCDMODE_INTERNAL); 
  Lcd_Display_Init();
  Lcd_Init();
  Lcd_SetContrast(contrast);
}

void LcdBlink(int blink)
{
  if(blink)
    AT91C_BASE_SLCDC -> SLCDC_DR = AT91C_SLCDC_DISPMODE_BLINK|(64 <<  8);
  else
    AT91C_BASE_SLCDC -> SLCDC_DR = AT91C_SLCDC_DISPMODE_NORMAL|(64 <<  8);
}

