//  Copyright (c) 2007 Hewlett-Packard development company llc
//
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are met:
//
//  - Redistributions of source code must retain the above copyright notice,
//  this list of conditions and the disclaimer below.
//
//  - Redistributions in binary form must reproduce the above copyright notice,
//  this list of conditions and the disclaimer below in the documentation and/or
//  other materials provided with the distribution. Unless a specific license is granted
//  to te licencee
//
//  HP's name may not be used to endorse or promote products derived from
//  this software without specific prior written permission.
//
//  DISCLAIMER:  THIS SOFTWARE IS PROVIDED BY HP "AS IS" AND ANY EXPRESS OR
//  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
//  DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
//  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
//  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
//  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
//  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//  ----------------------------------------------------------------------------
#include "ScreenHandeling.h"
#include "VirtualLCD.h"
#include "graphics.h"

unsigned int LcdData[20];
unsigned int SavedLcdData[20];

CVirtualLCD *g_pLCD;


typedef struct { int nbPoints; POINT points[58]; } TPoly  ;  
typedef struct { int nbPoly; TPoly poly[5]; } TSeg;

/*! Macros -
* 
*	These Macros are defined to create segments on a bitmap for CopyToClipboard feature
*	
*	
*
*
*/

#define square(x, y) { 1, { 4, {{x, y}, {x+18, y}, {x+18, y+18}, {x, y+18} } } }
#define matrixpoint(r, c) square(c*19+96, r*19+50) // the *19 represent the spacing between points and the +96/+50 the offset

#define matrixfirst33seg(r) matrixpoint(r, 0), matrixpoint(r, 1), matrixpoint(r, 2), matrixpoint(r, 3), matrixpoint(r, 4), matrixpoint(r, 5), matrixpoint(r, 6), matrixpoint(r, 7), matrixpoint(r, 8), matrixpoint(r, 9), matrixpoint(r, 10), matrixpoint(r, 11), matrixpoint(r, 12), matrixpoint(r, 13), matrixpoint(r, 14), matrixpoint(r, 15), matrixpoint(r, 16), matrixpoint(r, 17), matrixpoint(r, 18), matrixpoint(r, 19),  matrixpoint(r, 20), matrixpoint(r, 21), matrixpoint(r, 22), matrixpoint(r, 23), matrixpoint(r, 24), matrixpoint(r, 25), matrixpoint(r, 26), matrixpoint(r, 27), matrixpoint(r, 28), matrixpoint(r, 29), matrixpoint(r, 30), matrixpoint(r, 31), matrixpoint(r, 32)
#define matrixLast6Seg(c) matrixpoint(5, (c+33)), matrixpoint(4, (c+33)), matrixpoint(3, (c+33)), matrixpoint(2, (c+33)), matrixpoint(1, (c+33)), matrixpoint(0, (c+33))

#define SignEqualsTo {1, { 4, {{0,0},{0,0},{0,0},{0,0}} } } 
#define SignArrow	{1, { 7, {  {1083, 39 }, {1091, 39 }, {1091, 62 }, {1096, 62 }, {1087, 71 }, {1078, 62 }, {1083, 62 } }  } }

#define SignInput	{5, {{ 4, {{1130, 40 }, {1138, 40 }, {1138, 69 }, {1130, 69 } } }, \
{ 10, {  {1142, 40 }, {1151, 40 }, {1156, 52 }, {1156, 40 }, {1166, 40 }, {1166, 69 }, {1158, 69 }, {1151, 54 }, {1151, 69}, {1142, 69 } } } , \
{ 15, {  { 1170, 39 },{ 1184, 39 },{ 1191, 44 },{ 1191, 56 },{ 1185, 61 },{ 1178, 61 },{ 1178, 69 },{ 1170, 69 },{ 1170, 53 },{ 1183, 53 },{ 1186, 49 },{ 1182, 46 },{ 1177, 46 },{ 1177, 53 },{ 1170, 53 }} } , \
{ 14, {  {1195, 40 }, {1202, 40 }, {1202, 56 }, {1203, 59 }, {1205, 61 }, {1208, 61 }, {1209, 58 }, {1209, 40 }, {1217, 40 }, {1217, 62 }, {1214, 66 }, {1211, 68 }, {1200, 68 }, {1195, 62 } } } , \
{ 8, {  {1220, 40 }, {1242, 40 }, {1242, 48 }, {1235, 48 }, {1235, 69 }, {1227, 69 }, {1227, 48 }, {1220, 48 }} } } }
#define SignSto		{3, {{ 42, {{1204, 101 }, {1204, 104 }, {1206, 106 }, {1209, 108 }, {1212, 110 }, {1215, 110 }, {1219, 110 }, {1222, 107 }, {1225, 105 }, {1226, 102 }, {1226, 98 }, {1224, 95 }, {1221, 93 }, {1216, 92 }, {1213, 90 }, {1213, 89 }, {1213, 87 }, {1215, 86 }, {1216, 87 }, {1218, 89 }, {1219, 90 }, {1224, 90 }, {1225, 86 }, {1223, 84 }, {1221, 82 }, {1219, 81 }, {1218, 80 }, {1211, 80 }, {1209, 82 }, {1207, 83 }, {1205, 85 }, {1205, 88 }, {1205, 92 }, {1208, 95 }, {1212, 97 }, {1215, 99 }, {1218, 101 }, {1218, 102 }, {1215, 104 }, {1213, 103 }, {1212, 101 }, {1211, 100 } } } , \
{ 8, {  {1228, 81 }, {1248, 81 }, {1248, 88 }, {1241, 88 }, {1241, 109 }, {1235, 109}, {1235, 88 }, {1228, 88 } } } , \
{ 21, {  { 1259, 80 },{ 1268, 80 },{ 1272, 84 },{ 1275, 89 },{ 1275, 103 },{ 1272, 107 },{ 1268, 109 },{ 1258, 109 },{ 1255, 107 },{ 1252, 103 },{ 1265, 103 },{ 1268, 100 },{ 1268, 92 },{ 1266, 89 },{ 1260, 89 },{ 1258, 92 },{ 1258, 100 },{ 1260, 103 },{ 1252, 103 },{ 1252, 89 },{1255,82} }  }  }}
#define SignRad		{3, {{ 18, {{ 1082, 123 },{ 1098, 123 },{ 1101, 127 },{ 1101, 135 },{ 1098, 137 },{ 1101, 139 },{ 1101, 151 },{ 1094, 151 },{ 1094, 139 },{ 1089, 139 },{ 1089, 151 },{ 1082, 151 },{ 1082, 135 },{ 1094, 135 },{ 1094, 128 },{ 1089, 128 },{ 1089, 135 },{ 1082, 135 }, } } , \
{ 15, {  {1105, 151 }, {1110, 123 }, {1124, 123 }, {1129, 151 }, {1119, 151 }, {1118, 146 }, {1118, 141 }, {1119, 140 }, {1117, 131 }, {1114, 141 }, {1119, 141 }, {1119, 146 }, {1115, 145 }, {1115, 151 }, {1106, 151 } } } , \
{ 13, {  {1132, 123 }, {1148, 123 }, {1154, 128 }, {1154, 147 }, {1148, 151 }, {1132, 150 }, {1132, 145 }, {1141, 144 }, {1145, 144 }, {1145, 130 }, {1141, 128 }, {1141, 144 },{1132, 145 } }  }  } }
#define SignRcl		{3, {{ 21, {{1325, 81 }, {1342, 81 }, {1347, 86 }, {1347, 92 }, {1343, 95 }, {1347, 100 }, {1347, 109 }, {1340, 109 }, {1340, 102 }, {1339, 99 }, {1333, 99 }, {1333, 109 }, {1325, 109 }, {1325, 94 }, {1334, 94 }, {1339, 94 }, {1341, 91 }, {1339, 86 }, {1334, 86 }, {1334, 94}, {1325, 94 } } },  \
{ 25, {  {1373, 90 }, {1372, 86 }, {1369, 82 }, {1365, 81 }, {1359, 80 }, {1355, 83 }, {1352, 87 }, {1350, 93 }, {1351, 99 }, {1353, 105 }, {1357, 109 }, {1363, 109 }, {1367, 109 }, {1370, 107 }, {1374, 102 }, {1374, 99 }, {1367, 98 }, {1365, 102 }, {1362, 103 }, {1360, 101 }, {1358, 96 }, {1358, 91 }, {1360, 88 }, {1363, 88 }, {1366, 90 } }  }, \
{ 6, {  {1377, 80 }, {1386, 80}, {1386, 102 }, {1395, 102 }, {1395, 110 }, {1377, 110 } } } }}
#define SignBat		{1, { 8, {{1334, 40 }, {1334, 69 }, {1387, 69 }, {1387, 61 }, {1396, 61 }, {1397, 48 }, {1387, 47 }, {1387, 39 }, }  } }
#define SignBeg		{3, {25, {{ 1082, 81 },{ 1098, 81 },{ 1102, 87 },{ 1102, 93 },{ 1100, 96 },{ 1102, 99 },{ 1102, 104 },{ 1098, 110 },{ 1082, 110 },{ 1082, 104 },{ 1096, 104 },{ 1097, 101 },{ 1097, 97 },{ 1090, 97 },{ 1088, 100 },{ 1090, 104 },{ 1082, 104 },{ 1082, 92 },{ 1096, 92 },{ 1097, 89 },{ 1097, 86 },{ 1090, 86 },{ 1088, 89 },{ 1089, 92 },{ 1082, 92 }   }, \
{ 12, {  { 1107, 80 },{ 1127, 80 },{ 1127, 87 },{ 1114, 87 },{ 1114, 92 },{ 1125, 92 },{ 1125, 97 },{ 1113, 97 },{ 1113, 103 },{ 1127, 103 },{ 1127, 110 },{ 1107, 110 } } } , \
{ 25, {  { 1151, 90 },{ 1151, 86 },{ 1146, 81 },{ 1137, 81 },{ 1131, 87 },{ 1131, 102 },{ 1135, 109 },{ 1145, 109 },{ 1145, 106 },{ 1149, 106 },{ 1149, 109 },{ 1153, 109 },{ 1153, 95 },{ 1143, 95 },{ 1143, 100 },{ 1147, 100 },{ 1147, 103 },{ 1144, 103 },{ 1144, 104 },{ 1139, 104 },{ 1136, 101 },{ 1136, 92 },{ 1140, 88 },{ 1145, 88 },{ 1145, 90 } }  } }}
#define SignRpn		{3, { { 18, {{ 1326, 123 },{ 1342, 123 },{ 1347, 128 },{ 1347, 132 },{ 1343, 137 },{ 1346, 140 },{ 1346, 151 },{ 1339, 151 },{ 1339, 139 },{ 1334, 139 },{ 1334, 151 },{ 1326, 151 },{ 1326, 135 },{ 1339, 135 },{ 1339, 128 },{ 1334, 128 },{ 1334, 135 },{ 1326, 135 } } }, \
{ 21, {  {1361, 141 }, {1361, 151 }, {1352, 151 }, {1352, 131 }, {1361, 131 }, {1361, 135 }, {1363, 135 }, {1364, 134 }, {1365, 129 }, {1363, 127 }, {1361, 128 }, {1361, 131 }, {1352, 131 }, {1352, 123 }, {1367, 123 }, {1370, 125 }, {1372, 127 }, {1372, 132 }, {1371, 138 }, {1368, 140 }, {1361, 140 } } }, \
{ 10, {  {1376, 124 }, {1384, 124 }, {1389, 134 }, {1389, 124}, {1397, 124 }, {1397, 151 }, {1389, 151 }, {1384, 139 }, {1383, 151 }, {1376, 151 } } } } } 
#define SignL2		{2, {{ 4, {{1276, 45 }, {1302, 45 }, {1302, 52 }, {1276, 52 }}} , \
{ 4, {{1276, 57 }, {1302, 57 }, {1302, 64 }, {1276, 64 }}} }  } 


/* A */ #define NumberSegUp(n)	{ 1, { 6,{{(n-1)*89+105,	216}, {(n-1)*89+117,	212},		{(n-1)*89+146,	212}, {(n-1)*89+157,	216}, {(n-1)*89+146, 227}, {(n-1)*89+115, 227}	} } } // put definition there top segment
/* B */ #define NumberSegUR(n)	{ 1, { 6,{{(n-1)*89+161,	221}, {(n-1)*89+164,	227},		{(n-1)*89+163,	267}, {(n-1)*89+153,	275}, {(n-1)*89+146, 267}, {(n-1)*89+149, 232}	} } } // put definition there top segment
/* C */ #define NumberSegLR(n)  { 1, { 6,{{(n-1)*89+153,	282}, {(n-1)*89+160,	290},		{(n-1)*89+156,	329}, {(n-1)*89+151,	338}, {(n-1)*89+141, 327}, {(n-1)*89+144, 290}	} } } // put definition there top segment
/* D */ #define NumberSegBot(n) { 1, { 6,{{(n-1)*89+106,	330}, {(n-1)*89+138,	330},		{(n-1)*89+148,	341}, {(n-1)*89+136,  346}, {(n-1)*89+106, 346}, {(n-1)*89+95,  340}	} } } // put definition there top segment
/* E */ #define NumberSegLL(n)  { 1, { 6,{{(n-1)*89+100,	282}, {(n-1)*89+107,	290},		{(n-1)*89+104,  326}, {(n-1)*89+92,   338}, {(n-1)*89+89,  330}, {(n-1)*89+92,  289}		} } } // put definition there top segment
/* F */ #define NumberSegUL(n)	{ 1, { 6,{{(n-1)*89+101,	220}, {(n-1)*89+112,	232},		{(n-1)*89+110,  265}, {(n-1)*89+100,	275}, {(n-1)*89+93,  267}, {(n-1)*89+96,  230}	} } } // put definition there top segment
/* G */ #define NumberSegMid(n) { 1, { 6,{{(n-1)*89+104,	278}, {(n-1)*89+112,	271},		{(n-1)*89+143,	271}, {(n-1)*89+149,  278}, {(n-1)*89+143, 287}, {(n-1)*89+112, 287}	} } } // put definition there top segment

#define NumberSegDot(n)			{1,  { 15, {{(n-1)*89+160, 329 +10},  {(n-1)*89+160, 325 +10},  {(n-1)*89+162, 323 +10},  {(n-1)*89+163, 321 +10},  {(n-1)*89+164, 320 +10},  {(n-1)*89+167, 320 +10},  {(n-1)*89+169, 321 +10},  {(n-1)*89+172, 322 +10},  {(n-1)*89+173, 325 +10},  {(n-1)*89+173, 329 +10},  {(n-1)*89+171, 331 +10},  {(n-1)*89+169, 333 +10},  {(n-1)*89+165, 333 +10},  {(n-1)*89+163, 331 +10},  {(n-1)*89+161, 330 +10}}}}
#define NumberSegCom(n)			{1,  { 15, {{(n-1)*89+161, 345 +10},  {(n-1)*89+161, 342 +10},  {(n-1)*89+164, 338 +10},  {(n-1)*89+168, 338 +10},  {(n-1)*89+172, 341 +10},  {(n-1)*89+173, 347 +10},  {(n-1)*89+171, 350 +10},  {(n-1)*89+169, 353 +10},  {(n-1)*89+165, 355 +10},  {(n-1)*89+160, 355 +10},  {(n-1)*89+160, 355 +10},  {(n-1)*89+163, 353 +10},  {(n-1)*89+164, 351 +10},  {(n-1)*89+163, 349 +10},  {(n-1)*89+162, 347 +10} } } } 


#define NumberSegRightUp(n)		{ 1, { 6,{{(n-13)*72+1215, 185}, {(n-13)*72+1222, 182}, {(n-13)*72+1248, 182}, {(n-13)*72+1254, 185}, {(n-13)*72+1246, 193}, {(n-13)*72+1222, 193} } } }
#define NumberSegRightUR(n)		{ 1, { 6,{{(n-13)*72+1256, 187}, {(n-13)*72+1259, 192}, {(n-13)*72+1257, 217}, {(n-13)*72+1252, 223}, {(n-13)*72+1246, 217}, {(n-13)*72+1248, 195} } } }
#define NumberSegRightUL(n)		{ 1, { 6,{{(n-13)*72+1213, 187}, {(n-13)*72+1221, 195}, {(n-13)*72+1219, 217}, {(n-13)*72+1213, 222}, {(n-13)*72+1208, 217}, {(n-13)*72+1210, 192} } } }
#define NumberSegRightMid2(n, b)	{ 1, { 6,{{(n-13)*72+(b)+1214, 225}, {(n-13)*72+(b)+1220, 220}, {(n-13)*72+(b)+1244, 220}, {(n-13)*72+(b)+1249, 225}, {(n-13)*72+(b)+1242, 230}, {(n-13)*72+(b)+1220, 230} } } }
#define NumberSegRightMid(n)	NumberSegRightMid2(n ,0)
#define NumberSegRightLR(n)		{ 1, { 6,{{(n-13)*72+1250, 226}, {(n-13)*72+1256, 232}, {(n-13)*72+1253, 257}, {(n-13)*72+1250, 263}, {(n-13)*72+1242, 254}, {(n-13)*72+1244, 232} } } }
#define NumberSegRightLL(n)	  { 1, { 6,{{(n-13)*72+1212, 227}, {(n-13)*72+1217, 232}, {(n-13)*72+1215, 254}, {(n-13)*72+1207, 261}, {(n-13)*72+1204, 254}, {(n-13)*72+1207, 232} } } }
#define NumberSegRightBot(n)	{ 1, { 6,{{(n-13)*72+1209, 264}, {(n-13)*72+1217, 257}, {(n-13)*72+1240, 257}, {(n-13)*72+1247, 264}, {(n-13)*72+1242, 267}, {(n-13)*72+1215, 267} } } }

// Table for LCD emulation for CopyToClipboard feature
TSeg const table[400]=
{
	// populate the table using macros to help yourself and hard coding what is not practicle to put in macros (for example
	// COM 0
	matrixLast6Seg(8),		matrixfirst33seg(4),		SignArrow,

	// COM 1
	matrixLast6Seg(9),		matrixfirst33seg(5),		
	{ 2, {{ 4, { {949, 89 }, {997, 89 }, {997, 101 }, {949, 101 } } },
	{  4, { {949, 110 }, {997, 110 }, {997, 122 }, {949, 122 } } } } }, // Large equal sign...


	// COM 2
	matrixLast6Seg(0),		NumberSegUL(1),				NumberSegUp(1), 
	NumberSegUR(1),			NumberSegUp(2),				NumberSegUL(3),
	NumberSegUp(3),			NumberSegUR(3),				NumberSegUL(4),
	NumberSegUp(4),			NumberSegUL(5),				NumberSegUp(5),
	NumberSegUR(5),			NumberSegUp(6),				NumberSegUL(7),
	NumberSegUp(7),			NumberSegUR(7),				NumberSegUL(8),
	NumberSegUp(8),			NumberSegUL(9),				NumberSegUp(9),
	NumberSegUR(9),			NumberSegUp(10),			NumberSegUL(11),
	NumberSegUp(11),		NumberSegUR(11),			NumberSegUL(12),
	NumberSegUp(12),		NumberSegRightUL(13),		NumberSegRightUp(13),
	NumberSegRightUL(14),	NumberSegRightUp(14),		NumberSegRightUL(15),
	SignRpn,
	/*360*/ {3, { { 58, {{1202, 132 }, {1202, 127 }, {1202, 125 }, {1204, 123 }, {1207, 122 }, {1211, 122 }, {1217, 122 }, {1220, 124 }, {1222, 126 }, {1223, 128 }, {1223, 131 }, {1222, 133 }, {1221, 134 }, {1220, 135 }, {1221, 136 }, {1222, 137 }, {1223, 139 }, {1223, 142 }, {1223, 147 }, {1222, 149 }, {1220, 150 }, {1218, 152 }, {1214, 152 }, {1209, 152 }, {1207, 152 }, {1205, 150 }, {1203, 149 }, {1203, 146 }, {1201, 145 }, {1201, 142 }, {1203, 140 }, {1205, 140 }, {1207, 140 }, {1209, 140 }, {1211, 142 }, {1211, 144 }, {1211, 146 }, {1212, 146 }, {1214, 147 }, {1215, 145 }, {1215, 141 }, {1214, 139 }, {1212, 138 }, {1210, 138 }, {1209, 137 }, {1209, 134 }, {1210, 134 }, {1212, 134 }, {1213, 133 }, {1215, 132 }, {1216, 130 }, {1215, 128 }, {1213, 127 }, {1211, 127 }, {1211, 129 }, {1211, 131 }, {1210, 132 }, {1203, 132 } } }, 
	{ 40, {{1249, 130 }, {1248, 126 }, {1247, 125 }, {1245, 124 }, {1242, 123 }, {1236, 123 }, {1234, 123 }, {1232, 124 }, {1230, 126 }, {1228, 127 }, {1228, 130 }, {1228, 145 }, {1229, 147 }, {1231, 149 }, {1233, 151 }, {1234, 152 }, {1244, 152 }, {1247, 150 }, {1249, 148 }, {1250, 147 }, {1249, 142 }, {1241, 142 }, {1241, 146 }, {1238, 146 }, {1237, 145 }, {1237, 139 }, {1239, 138 }, {1241, 138 }, {1241, 142 }, {1248, 142 }, {1248, 135 }, {1246, 134 }, {1244, 133 }, {1239, 133 }, {1237, 133 }, {1236, 132 }, {1237, 129 }, {1238, 127 }, {1241, 128 }, {1242, 130 } } },
	{ 19, {{1255, 128 }, {1258, 124 }, {1261, 123 }, {1271, 123 }, {1273, 125 }, {1276, 128 }, {1276, 146 }, {1274, 149 }, {1270, 151 }, {1260, 151 }, {1255, 147 }, {1262, 147 }, {1265, 147 }, {1267, 147}, {1267, 129 }, {1265, 127 }, {1262, 129 }, {1262, 147}, {1255, 147 } }  } }},

	// COM 3
	matrixLast6Seg(1),		NumberSegLL(1),				NumberSegMid(1),
	NumberSegUL(2),			NumberSegMid(2),			NumberSegUR(2),
	NumberSegMid(3),		NumberSegLR(3),				NumberSegMid(4),
	NumberSegUR(4),			NumberSegLL(5),				NumberSegMid(5),
	NumberSegUL(6),			NumberSegMid(6),			NumberSegUR(6),
	NumberSegMid(7),		NumberSegLR(7),				NumberSegMid(8),
	NumberSegUR(8),			NumberSegLL(9),				NumberSegMid(9),
	NumberSegUL(10),		NumberSegMid(10),			NumberSegUR(10),
	NumberSegMid(11),		NumberSegLR(11),			NumberSegMid(12),
	NumberSegUR(12),		NumberSegRightMid2(12, 17),
	NumberSegRightUR(13),	NumberSegRightMid(14),		NumberSegRightUR(14),
	NumberSegRightMid(15),	NumberSegRightUp(15),		SignRad,

	// COM 4
	matrixLast6Seg(2),		NumberSegBot(1),			NumberSegLR(1),	
	NumberSegLL(2),			NumberSegBot(2),			NumberSegLR(2),
	NumberSegLL(3),			NumberSegDot(3),			NumberSegLL(4),
	NumberSegLR(4),			NumberSegDot(4),			NumberSegLR(5),
	NumberSegLL(6),			NumberSegBot(6),			NumberSegLR(6),
	NumberSegLL(7),			NumberSegDot(7),			NumberSegLL(8),
	NumberSegLR(8),			NumberSegDot(8),			NumberSegLR(9),
	NumberSegLL(10),		NumberSegBot(10),			NumberSegLR(10),
	NumberSegLL(11),		NumberSegDot(11),			NumberSegLL(12),
	NumberSegLR(12),		NumberSegRightLL(13),		NumberSegRightMid(13),
	NumberSegRightLL(14),	NumberSegRightLR(14),		NumberSegRightLL(15),
	NumberSegRightUR(15),	SignRcl,

	// COM 5
	matrixLast6Seg(3),		NumberSegMid(0),					NumberSegCom(1),
	NumberSegDot(1),		NumberSegCom(2),			NumberSegDot(2),
	NumberSegBot(3),		NumberSegCom(3),			NumberSegBot(4),
	NumberSegCom(4),		NumberSegBot(5),			NumberSegCom(5),
	NumberSegDot(5),		NumberSegCom(6),			NumberSegDot(6),
	NumberSegBot(7),		NumberSegCom(7),			NumberSegBot(8),
	NumberSegCom(8),		NumberSegBot(9),			NumberSegCom(9),
	NumberSegDot(9),		NumberSegCom(10),			NumberSegDot(10),
	NumberSegBot(11),		NumberSegCom(11),			NumberSegBot(12),
	NumberSegCom(12),		NumberSegDot(12),			NumberSegRightBot(13),
	NumberSegRightLR(13),	NumberSegRightBot(14),		NumberSegRightBot(15),
	NumberSegRightLR(15),	SignSto,

	// COM 6
	matrixLast6Seg(4),		matrixfirst33seg(0),		SignBeg,
	// COM 7
	matrixLast6Seg(5),		matrixfirst33seg(1),		SignBat,
	// COM 8
	matrixLast6Seg(6),		matrixfirst33seg(2),		SignL2,
	// COM 9
	matrixLast6Seg(7),		matrixfirst33seg(3),		SignInput,
};


/*! Macros -
* 
*	These Macros are defined to create segments on a bitmap for LCD emulation of Euro Calculator
*	
*	
*
*
*/

#define square_LCD(x, y) { 1, { 4, {{x, y}, {x+3, y}, {x+3, y+3}, {x, y+3} } } }
#define matrixpoint_LCD(r, c) square_LCD(c*4+22, r*4+11) // the *19 represent the spacing between points and the +96/+50 the offset

#define matrixfirst33seg_LCD(r) matrixpoint_LCD(r, 0), matrixpoint_LCD(r, 1), matrixpoint_LCD(r, 2), matrixpoint_LCD(r, 3), matrixpoint_LCD(r, 4), matrixpoint_LCD(r, 5), matrixpoint_LCD(r, 6), matrixpoint_LCD(r, 7), matrixpoint_LCD(r, 8), matrixpoint_LCD(r, 9), matrixpoint_LCD(r, 10), matrixpoint_LCD(r, 11), matrixpoint_LCD(r, 12), matrixpoint_LCD(r, 13), matrixpoint_LCD(r, 14), matrixpoint_LCD(r, 15), matrixpoint_LCD(r, 16), matrixpoint_LCD(r, 17), matrixpoint_LCD(r, 18), matrixpoint_LCD(r, 19),  matrixpoint_LCD(r, 20), matrixpoint_LCD(r, 21), matrixpoint_LCD(r, 22), matrixpoint_LCD(r, 23), matrixpoint_LCD(r, 24), matrixpoint_LCD(r, 25), matrixpoint_LCD(r, 26), matrixpoint_LCD(r, 27), matrixpoint_LCD(r, 28), matrixpoint_LCD(r, 29), matrixpoint_LCD(r, 30), matrixpoint_LCD(r, 31), matrixpoint_LCD(r, 32)
#define matrixLast6Seg_LCD(c) matrixpoint_LCD(5, (c+33)), matrixpoint_LCD(4, (c+33)), matrixpoint_LCD(3, (c+33)), matrixpoint_LCD(2, (c+33)), matrixpoint_LCD(1, (c+33)), matrixpoint_LCD(0, (c+33))


#define SignArrow_LCD	{1, { 7, {  { 250, 9 },{ 252, 9 },{ 252, 13 },{ 254, 13 },{ 251, 17 },{ 248, 13 },{ 250, 13 } }  } }
#define SignInput_LCD	{5, {{ 3, {{262, 9 }, {262, 16 }, {262, 9 } } }, \
{ 12, {  {264, 16 }, {264, 9 }, {265, 9 }, {265, 11 }, {266, 12 }, {268, 16 }, {268, 9 }, {268, 16 }, {267, 14 }, {266, 12 }, {264, 9 }, {264, 16 } } } , \
{ 11, {  {271, 16 }, {271, 9 }, {273, 9 }, {274, 10 }, {274, 13 }, {271, 13 }, {274, 13 }, {274, 10 }, {273, 9 }, {271, 9 }, {271, 16 }} } , \
{ 11, {  {276, 9 }, {276, 15 }, {277, 16 }, {279, 16 }, {280, 15 }, {280, 9 }, {280, 15 }, {279, 16 }, {277, 16 }, {276, 15 }, {276, 9 } } } , \
{ 6, {  {282, 9 }, {286, 9 }, {284, 9 }, {284, 16 }, {284, 9 }, {282, 9 }} } } }
#define SignSto_LCD		{3, {{ 23, {{282, 20 }, {281, 19 }, {280, 19 }, {279, 20 }, {279, 21 }, {280, 22 }, {281, 22 }, {282, 23 }, {282, 24 }, {281, 25 }, {280, 25 }, {279, 24 }, {280, 25 }, {281, 25 }, {282, 24 }, {282, 23 }, {281, 22 }, {280, 22 }, {279, 21 }, {279, 20 }, {280, 19 }, {281, 19 }, {282, 20 } } } , \
{ 6, {  {284, 19 }, {288, 19 }, {286, 19 }, {286, 25 }, {286, 19 }, {284, 19 } } } , \
{ 25, {  {291, 19 }, {292, 19 }, {293, 20 }, {294, 21 }, {294, 23 }, {293, 24 }, {292, 25 }, {291, 25 }, {290, 24 }, {289, 23 }, {289, 21 }, {290, 20 }, {291, 19 }, {290, 20 }, {289, 21 }, {289, 23 }, {290, 24 }, {291, 25 }, {292, 25 }, {293, 24 }, {294, 23 }, {294, 21 }, {293, 20 }, {292, 19 }, {291, 19 } }  }  }}
#define SignRad_LCD		{3, {{ 24, {{250, 28 }, {253, 28 }, {254, 29 }, {254, 30 }, {254, 31 }, {253, 31 }, {252, 32 }, {253, 33 }, {254, 33 }, {254, 35 }, {254, 33 }, {254, 33 }, {253, 33 }, {252, 32 }, {253, 31 }, {254, 31 }, {254, 29 }, {253, 28 }, {250, 28 }, {250, 32 }, {252, 32 }, {250, 32 }, {250, 35 }, {250, 28 }, } } , \
{ 11, {  {256, 35 }, {256, 28 }, {260, 28 }, {260, 35 }, {260, 32 }, {256, 32 }, {260, 32 }, {260, 28 }, {256, 28 }, {256, 32 }, {256, 35 } } } , \
{ 15, {  {262, 28 }, {265, 28 }, {266, 30 }, {266, 33 }, {265, 35 }, {265, 35 }, {262, 35 }, {265, 35 }, {265, 34 }, {266, 33 }, {266, 30 }, {265, 29 }, {265, 28 }, {262, 28 }, {262, 35 } }  }  } }
#define SignRcl_LCD		{3, {{ 19, {{306, 19 }, {309, 19 }, {310, 20 }, {310, 21 }, {309, 22 }, {307, 22 }, {309, 22 }, {310, 23 }, {310, 25 }, {310, 23 }, {309, 22 }, {307, 22 }, {309, 22 }, {310, 21 }, {310, 20 }, {309, 19 }, {306, 19 }, {306, 25 }, {306, 19 } } },  \
{ 19, {  {316, 21 }, {316, 20 }, {315, 19 }, {314, 19 }, {313, 20 }, {313, 24 }, {314, 25 }, {315, 25 }, {316, 24 }, {316, 23 }, {316, 24 }, {315, 25 }, {314, 25 }, {313, 24 }, {313, 20 }, {314, 19 }, {315, 19 }, {316, 20 }, {316, 21 } }  }, \
{ 5, {  {319, 19 }, {319, 25 }, {322, 25 }, {319, 25 }, {319, 19 } } } }}
#define SignBat_LCD		{1, { 8, {{1334, 40 }, {1334, 69 }, {1387, 69 }, {1387, 61 }, {1396, 61 }, {1397, 48 }, {1387, 47 }, {1387, 39 }, }  } }
#define SignBeg_LCD		{3, {23, {{250, 19 }, {253, 19 }, {254, 20 }, {254, 21 }, {253, 22 }, {250, 22 }, {253, 22 }, {254, 23 }, {254, 24 }, {253, 25 }, {250, 25 }, {250, 19 }, {250, 25 }, {253, 25 }, {254, 24 }, {254, 23 }, {253, 22 }, {250, 22 }, {253, 22 }, {254, 21 }, {254, 20 }, {253, 19 }, {250, 19 }  }, \
{ 9, {  {256, 19 }, {259, 19 }, {256, 19 }, {256, 22 }, {259, 22 }, {256, 22 }, {256, 25 }, {259, 25 }, {256, 25 } } } , \
{ 24, {  {265, 20 }, {264, 19 }, {263, 19 }, {262, 20 }, {261, 21 }, {261, 23 }, {262, 24 }, {263, 25 }, {264, 25 }, {265, 25 }, {265, 24 }, {265, 23 }, {264, 23 }, {265, 23 }, {265, 24 }, {265, 25 }, {263, 25 }, {262, 24 }, {261, 23 }, {261, 21 }, {262, 20 }, {263, 19 }, {264, 19 }, {265, 20 } }  } }}
#define SignRpn_LCD		{3, { { 19, {{306, 35 }, {306, 28 }, {309, 28 }, {310, 29 }, {310, 31 }, {309, 32 }, {306, 32 }, {309, 32 }, {310, 33 }, {310, 35 }, {310, 33 }, {309, 32 }, {306, 32 }, {309, 32 }, {310, 31 }, {310, 29 }, {309, 28 }, {306, 28 }, {306, 35 } } }, \
{ 13, {  {312, 35 }, {312, 28 }, {315, 28 }, {316, 29 }, {316, 31 }, {315, 32 }, {312, 32 }, {315, 32 }, {316, 31 }, {316, 29 }, {315, 28 }, {312, 28 }, {312, 35 } } }, \
{ 9, {  {318, 35 }, {318, 28 }, {318, 29 }, {322, 34 }, {322, 35 }, {322, 28 }, {322, 35 }, {318, 28 }, {318, 35 } } } } } 
#define SignL2_LCD		{2, {{ 3, {{295, 11 }, {300, 11 }, {295, 11 }}} , \
{ 3, {{295, 14 }, {300, 14 }, {295, 14 }}} }  } 

#define VShift 5
/* A */ #define NumberSegUp_LCD(n)	{ 1, { 6,{{(n-1)*20+25, 50 +VShift},  {(n-1)*20+26, 49 +VShift},  {(n-1)*20+35, 49 +VShift},  {(n-1)*20+36, 50 +VShift},  {(n-1)*20+33, 53 +VShift},  {(n-1)*20+27, 53 +VShift}	} } } // put definition there top segment
/* B */ #define NumberSegUR_LCD(n)	{ 1, { 6,{{(n-1)*20+35, 54 +VShift},  {(n-1)*20+37, 52 +VShift},  {(n-1)*20+38, 54 +VShift},  {(n-1)*20+38, 61 +VShift},  {(n-1)*20+37, 63 +VShift},  {(n-1)*20+35, 61 +VShift}	} } } // put definition there top segment
/* C */ #define NumberSegLR_LCD(n)  { 1, { 9,{{(n-1)*20+34, 68 +VShift},  {(n-1)*20+35, 67 +VShift},  {(n-1)*20+36, 66 +VShift},  {(n-1)*20+37, 67 +VShift},  {(n-1)*20+37, 76 +VShift},  {(n-1)*20+36, 77 +VShift},  {(n-1)*20+35, 78 +VShift},  {(n-1)*20+34, 77 +VShift},  {(n-1)*20+34, 70 +VShift}	} } } // put definition there top segment
/* D */ #define NumberSegBot_LCD(n) { 1, { 7,{{(n-1)*20+23, 79 +VShift},  {(n-1)*20+25, 77 +VShift},  {(n-1)*20+31, 77 +VShift},  {(n-1)*20+32, 78 +VShift},  {(n-1)*20+33, 79 +VShift},  {(n-1)*20+32, 80 +VShift},  {(n-1)*20+24, 80 +VShift}	} } } // put definition there top segment
/* E */ #define NumberSegLL_LCD(n)  { 1, { 6,{{(n-1)*20+21, 68 +VShift},  {(n-1)*20+21, 75 +VShift},  {(n-1)*20+22, 77 +VShift},  {(n-1)*20+24, 75 +VShift},  {(n-1)*20+24, 68 +VShift},  {(n-1)*20+22, 66 +VShift}	} } } // put definition there top segment
/* F */ #define NumberSegUL_LCD(n)	{ 1, { 6,{{(n-1)*20+23, 62 +VShift},  {(n-1)*20+25, 60 +VShift},  {(n-1)*20+25, 54 +VShift},  {(n-1)*20+23, 52 +VShift},  {(n-1)*20+22, 53 +VShift},  {(n-1)*20+22, 61 +VShift}	} } } // put definition there top segment
/* G */ #define NumberSegMid_LCD(n) { 1, { 8,{{(n-1)*20+24, 64 +VShift},  {(n-1)*20+24, 65 +VShift},  {(n-1)*20+25, 66 +VShift},  {(n-1)*20+33, 66 +VShift},  {(n-1)*20+34, 65 +VShift},  {(n-1)*20+34, 64 +VShift},  {(n-1)*20+33, 63 +VShift},  {(n-1)*20+25, 63 +VShift}	} } } // put definition there top segment

#define NumberSegDot_LCD(n)			{1,  { 9, {{(n-1)*20+39, 74 +3+VShift},   {(n-1)*20+40, 74 +3+VShift},   {(n-1)*20+41, 75 +3+VShift},   {(n-1)*20+41, 76 +3+VShift},   {(n-1)*20+40, 77 +3+VShift},   {(n-1)*20+39, 77 +3+VShift},   {(n-1)*20+38, 76 +3+VShift},   {(n-1)*20+38, 75 +3+VShift},   {(n-1)*20+39, 74 +3+VShift}}}}
#define NumberSegCom_LCD(n)			{1,  { 15, {{(n-1)*20+39, 79 +3+VShift},   {(n-1)*20+40, 79 +3+VShift},   {(n-1)*20+41, 80 +3+VShift},   {(n-1)*20+41, 81 +3+VShift},   {(n-1)*20+40, 82 +3+VShift},   {(n-1)*20+39, 82 +3+VShift},   {(n-1)*20+38, 83 +3+VShift},   {(n-1)*20+37, 83 +3+VShift},   {(n-1)*20+37, 83 +3+VShift},   {(n-1)*20+37, 82 +3+VShift},   {(n-1)*20+38, 82 +3+VShift},   {(n-1)*20+38, 81 +3+VShift},   {(n-1)*20+38, 80 +3+VShift},   {(n-1)*20+38, 79 +3+VShift},   {(n-1)*20+38, 79+3 +VShift} } } } 

/* A */ #define NumberSegRightUp_LCD(n)		{ 1, { 6,{{(n-13)*17+282, 43 +VShift},  {(n-13)*17+283, 42 +VShift},  {(n-13)*17+287, 42 +VShift},  {(n-13)*17+288, 43 +VShift},  {(n-13)*17+287, 44 +VShift},  {(n-13)*17+283, 44 +VShift} } } }
/* B */ #define NumberSegRightUR_LCD(n)		{ 1, { 6,{{(n-13)*17+289, 51 +VShift},  {(n-13)*17+290, 50 +VShift},  {(n-13)*17+290, 46 +VShift},  {(n-13)*17+289, 45 +VShift},  {(n-13)*17+288, 46 +VShift},  {(n-13)*17+288, 49 +VShift} } } }
/* C */ #define NumberSegRightLR_LCD(n)		{ 1, { 6,{{(n-13)*17+289, 53 +VShift},  {(n-13)*17+288, 54 +VShift},  {(n-13)*17+288, 59 +VShift},  {(n-13)*17+289, 60 +VShift},  {(n-13)*17+290, 59 +VShift},  {(n-13)*17+290, 54 +VShift} } } }
/* D */ #define NumberSegRightBot_LCD(n)	{ 1, { 6,{{(n-13)*17+281, 61 +VShift},  {(n-13)*17+282, 60 +VShift},  {(n-13)*17+286, 60 +VShift},  {(n-13)*17+287, 61 +VShift},  {(n-13)*17+286, 62 +VShift},  {(n-13)*17+282, 62 +VShift} } } }
/* E */ #define NumberSegRightLL_LCD(n)	    { 1, { 6,{{(n-13)*17+280, 53 +VShift},  {(n-13)*17+279, 54 +VShift},  {(n-13)*17+279, 59 +VShift},  {(n-13)*17+280, 60 +VShift},  {(n-13)*17+281, 59 +VShift},  {(n-13)*17+281, 54 +VShift} } } }
/* F */ #define NumberSegRightUL_LCD(n)		{ 1, { 6,{{(n-13)*17+280, 51 +VShift},  {(n-13)*17+279, 50 +VShift},  {(n-13)*17+279, 45 +VShift},  {(n-13)*17+280, 44 +VShift},  {(n-13)*17+281, 45 +VShift},  {(n-13)*17+281, 50 +VShift} } } }
/* G */ #define NumberSegRightMid2_LCD(n, b){ 1, { 6,{{(n-13)*17+282, 52 +VShift},  {(n-13)*17+283, 51 +VShift},  {(n-13)*17+286, 51 +VShift},  {(n-13)*17+287, 52 +VShift},  {(n-13)*17+286, 53 +VShift},  {(n-13)*17+283, 53 +VShift} } } }
#define NumberSegRightMid_LCD(n)	NumberSegRightMid2_LCD(n ,0)

// Table for LCD emulation  
TSeg const table_LCD[400]=
{
	// populate the table using macros to help yourself and hard coding what is not practicle to put in macros (for example
	
	// COM 0
	matrixLast6Seg_LCD(8),		matrixfirst33seg_LCD(4),		SignArrow_LCD,

	// COM 1
	matrixLast6Seg_LCD(9),		matrixfirst33seg_LCD(5),	
	{ 2, {{ 4, { { 219, 20 },{ 230, 20 },{ 230, 23 },{ 219, 23 } } },
	{  4, { { 219, 26 },{ 230, 26 },{ 230, 29 },{ 219, 29 }, } } } }, // Large equal sign...
	// COM 2
	matrixLast6Seg_LCD(0),		NumberSegUL_LCD(1),				NumberSegUp_LCD(1), 
	NumberSegUR_LCD(1),			NumberSegUp_LCD(2),				NumberSegUL_LCD(3),
	NumberSegUp_LCD(3),			NumberSegUR_LCD(3),				NumberSegUL_LCD(4),
	NumberSegUp_LCD(4),			NumberSegUL_LCD(5),				NumberSegUp_LCD(5),
	NumberSegUR_LCD(5),			NumberSegUp_LCD(6),				NumberSegUL_LCD(7),
	NumberSegUp_LCD(7),			NumberSegUR_LCD(7),				NumberSegUL_LCD(8),
	NumberSegUp_LCD(8),			NumberSegUL_LCD(9),				NumberSegUp_LCD(9),
	NumberSegUR_LCD(9),			NumberSegUp_LCD(10),			NumberSegUL_LCD(11),
	NumberSegUp_LCD(11),		NumberSegUR_LCD(11),			NumberSegUL_LCD(12),
	NumberSegUp_LCD(12),		NumberSegRightUL_LCD(13),		NumberSegRightUp_LCD(13),
	NumberSegRightUL_LCD(14),	NumberSegRightUp_LCD(14),		NumberSegRightUL_LCD(15),
	SignRpn_LCD,
	/*360*/ {3, { { 29, {{278, 30 }, {278, 29 }, {279, 28 }, {280, 28 }, {281, 28 }, {282, 29 }, {282, 31 }, {280, 31 }, {280, 32 }, {281, 32 }, {282, 32 }, {282, 34 }, {281, 35 }, {279, 35 }, {278, 34 }, {278, 33 }, {278, 34 }, {279, 35 }, {281, 35 }, {282, 34 }, {282, 32 }, {280, 32 }, {280, 31 }, {282, 31 }, {282, 29 }, {281, 28 }, {279, 28 }, {278, 29 }, {278, 30 } } }, 
	{ 21, {{288, 30 }, {288, 29 }, {287, 28 }, {285, 28 }, {284, 29 }, {284, 34 }, {285, 35 }, {287, 35 }, {288, 34 }, {288, 32 }, {284, 32 }, {288, 32 }, {288, 34 }, {287, 35 }, {285, 35 }, {284, 34 }, {284, 29 }, {285, 28 }, {287, 28 }, {288, 29 }, {288, 30 } } },
	{ 17, {{291, 28 }, {293, 28 }, {294, 29 }, {294, 30 }, {294, 34 }, {293, 35 }, {291, 35 }, {290, 34 }, {290, 30 }, {290, 29 }, {290, 34 }, {291, 35 }, {293, 35 }, {294, 34 }, {294, 29 }, {293, 28 }, {291, 28 } }  } }},

	// COM 3
	matrixLast6Seg_LCD(1),		NumberSegLL_LCD(1),				NumberSegMid_LCD(1),
	NumberSegUL_LCD(2),			NumberSegMid_LCD(2),			NumberSegUR_LCD(2),
	NumberSegMid_LCD(3),		NumberSegLR_LCD(3),				NumberSegMid_LCD(4),
	NumberSegUR_LCD(4),			NumberSegLL_LCD(5),				NumberSegMid_LCD(5),
	NumberSegUL_LCD(6),			NumberSegMid_LCD(6),			NumberSegUR_LCD(6),
	NumberSegMid_LCD(7),		NumberSegLR_LCD(7),				NumberSegMid_LCD(8),
	NumberSegUR_LCD(8),			NumberSegLL_LCD(9),				NumberSegMid_LCD(9),
	NumberSegUL_LCD(10),		NumberSegMid_LCD(10),			NumberSegUR_LCD(10),
	NumberSegMid_LCD(11),		NumberSegLR_LCD(11),			NumberSegMid_LCD(12),
	NumberSegUR_LCD(12),		NumberSegRightMid2_LCD(12, 17),
	NumberSegRightUR_LCD(13),	NumberSegRightMid_LCD(14),		NumberSegRightUR_LCD(14),
	NumberSegRightMid_LCD(15),	NumberSegRightUp_LCD(15),		SignRad_LCD,

	// COM 4
	matrixLast6Seg_LCD(2),		NumberSegBot_LCD(1),			NumberSegLR_LCD(1),	
	NumberSegLL_LCD(2),			NumberSegBot_LCD(2),			NumberSegLR_LCD(2),
	NumberSegLL_LCD(3),			NumberSegDot_LCD(3),			NumberSegLL_LCD(4),
	NumberSegLR_LCD(4),			NumberSegDot_LCD(4),			NumberSegLR_LCD(5),
	NumberSegLL_LCD(6),			NumberSegBot_LCD(6),			NumberSegLR_LCD(6),
	NumberSegLL_LCD(7),			NumberSegDot_LCD(7),			NumberSegLL_LCD(8),
	NumberSegLR_LCD(8),			NumberSegDot_LCD(8),			NumberSegLR_LCD(9),
	NumberSegLL_LCD(10),		NumberSegBot_LCD(10),			NumberSegLR_LCD(10),
	NumberSegLL_LCD(11),		NumberSegDot_LCD(11),			NumberSegLL_LCD(12),
	NumberSegLR_LCD(12),		NumberSegRightLL_LCD(13),		NumberSegRightMid_LCD(13),
	NumberSegRightLL_LCD(14),	NumberSegRightLR_LCD(14),		NumberSegRightLL_LCD(15),
	NumberSegRightUR_LCD(15),	SignRcl_LCD,

	// COM 5
	matrixLast6Seg_LCD(3),		NumberSegMid_LCD(0),			NumberSegCom_LCD(1),
	NumberSegDot_LCD(1),		NumberSegCom_LCD(2),			NumberSegDot_LCD(2),
	NumberSegBot_LCD(3),		NumberSegCom_LCD(3),			NumberSegBot_LCD(4),
	NumberSegCom_LCD(4),		NumberSegBot_LCD(5),			NumberSegCom_LCD(5),
	NumberSegDot_LCD(5),		NumberSegCom_LCD(6),			NumberSegDot_LCD(6),
	NumberSegBot_LCD(7),		NumberSegCom_LCD(7),			NumberSegBot_LCD(8),
	NumberSegCom_LCD(8),		NumberSegBot_LCD(9),			NumberSegCom_LCD(9),
	NumberSegDot_LCD(9),		NumberSegCom_LCD(10),			NumberSegDot_LCD(10),
	NumberSegBot_LCD(11),		NumberSegCom_LCD(11),			NumberSegBot_LCD(12),
	NumberSegCom_LCD(12),		NumberSegDot_LCD(12),			NumberSegRightBot_LCD(13),
	NumberSegRightLR_LCD(13),	NumberSegRightBot_LCD(14),		NumberSegRightBot_LCD(15),
	NumberSegRightLR_LCD(15),	SignSto_LCD,

	// COM 6
	matrixLast6Seg_LCD(4),		matrixfirst33seg_LCD(0),		SignBeg_LCD,
	// COM 7
	matrixLast6Seg_LCD(5),		matrixfirst33seg_LCD(1),		SignBat,
	// COM 8
	matrixLast6Seg_LCD(6),		matrixfirst33seg_LCD(2),		SignL2_LCD,
	// COM 9
	matrixLast6Seg_LCD(7),		matrixfirst33seg_LCD(3),		SignInput_LCD,

};

void UpdateScreenContent()
{
	unsigned long long *e= (unsigned long long *)AT91C_SLCDC_MEM;

	//  for (int i= 0; i<10; i++) e[i]= ~0LL;

	HBRUSH b= CreateSolidBrush(VIRTUAL_LCD_COLOR);
	RECT r={0, 0, CLPWidth, CLPHeight};
	FillRect(g_pLCD->m_hMemDC, &r, b);

	RECT r1={0, 0, LCDWidth, LCDHeight};
	FillRect(g_pLCD->m_hMyDC, &r1, b);
	
	DeleteObject(b);

	for (int r=0; r<10; r++){
		for (int c=0; c<40; c++){
			if ((e[r]&(1LL<<c))!=0LL)	{
				for (int n=0; n<table[r*40+c].nbPoly; n++){
					//For CopyToClipboard feature
					SetPolyFillMode(g_pLCD->m_hMemDC, WINDING);
					Polygon(g_pLCD->m_hMemDC, &table[r*40+c].poly[n].points[0], table[r*40+c].poly[n].nbPoints);
					//For LCD emulation
					SetPolyFillMode(g_pLCD->m_hMyDC, WINDING);
					Polygon(g_pLCD->m_hMyDC, &table_LCD[r*40+c].poly[n].points[0], table_LCD[r*40+c].poly[n].nbPoints);
				}  
			}
		}
	}
	g_pLCD->hpFinish();
}

