/*
Copyright (c) 2007 Hewlett-Packard development company llc
//
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are met:
//
//  - Redistributions of source code must retain the above copyright notice,
//  this list of conditions and the disclaimer below.
//
//  - Redistributions in binary form must reproduce the above copyright notice,
//  this list of conditions and the disclaimer below in the documentation and/or
//  other materials provided with the distribution. Unless a specific license is granted
//  to te licencee
//
//  - This software can only be used on a HP hardware. You are not permited to leaverage
//  any part of this software for products/project that do not work on hardware sold by HP
//
//  HP's name may not be used to endorse or promote products derived from
//  this software without specific prior written permission.
//
//  DISCLAIMER:  THIS SOFTWARE IS PROVIDED BY HP "AS IS" AND ANY EXPRESS OR
//  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
//  DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
//  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
//  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
//  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
//  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//  ----------------------------------------------------------------------------
file created by Cyrille de Brebisson
*/

#include "system.h"

// return true if the buffer is empty
int KeyBuffEmpty()
{
  char r= System.BufRead;
  char w= System.BufWrite;
  return r==w;
}

// reset the keyboard buffer
void KeyboardBufferClear()
{
  System.BufRead= System.BufWrite= 0;
}

// return a key from the key buffer. return -1 if buffer empty. does not remove the key from the buffer...
int KeyBuffPeek()
{
  if (KeyBuffEmpty()) return -1;
  return System.KeyBuffer[System.BufRead];
}

// return a key from the key buffer. return -1 if buffer empty
int KeyBuffGetKey()
{
  char r= System.BufRead;
  if (r==System.BufWrite) return -1;
  char key= System.KeyBuffer[r++];
  if (r==MAXKEYBUFSIZE) r=0;
  System.BufRead= r;
  return key;
}

// *****************************************************************************
// Keyboard buffer management

void AddKeyInBuffer(int k)
{
  char r= System.BufRead;
  char w= System.BufWrite;
  if (w!=r-1 && (w!=MAXKEYBUFSIZE || r!=0)) // if space in buffer
  {
    System.KeyBuffer[w]= k;                       // place key in buffer
    int t= w+1; if (t==MAXKEYBUFSIZE) t=0; // update write pointer
    System.BufWrite= t;
  }
}

void InitRS323(int speed)
{
  System.RS232Speed= speed;
  UpdateRS232();
}

#ifdef _ARM_
#include "dbgu.h"
__no_init TSystem System;
void UpdateRS232()
{
  if (System.RS232Speed==0)
  { //TODO: kill RS232...
  } else
    AT91F_Init_DBGU(RealCpuSpeed, System.RS232Speed);
}

#ifdef SETIO
void SetIO(int i)
{
  int r= 0;
  if (i&1)   r+= (1<<22);
  if (i&2)   r+= (1<<29);
  if (i&4)   r+= (1<<27);
  if (i&8)   r+= (1<<28);
  if (i&16)  r+= (1<<21);
  if (i&32)  r+= (1<<18);
  if (i&64)  r+= (1<<20);
  if (i&128) r+= (1<<19);
  r= r&SetIOMask;

  AT91C_BASE_PIOC->PIO_PER= SetIOMask; // take control
  AT91C_BASE_PIOC->PIO_OER = SetIOMask; // Configure in Output
  AT91C_BASE_PIOC->PIO_PPUDR = SetIOMask; // disable pullup
  AT91C_BASE_PIOC->PIO_SODR = r; // set pin to 1
  AT91C_BASE_PIOC->PIO_CODR = r^SetIOMask; // set other pins to 0
}
#endif
#else
void UpdateRS232() { }
TSystem System;
void SetCPUSpeed(int freq)
{
}
int BaseCpuSpeed= 0;
#endif

void initKeyBuffer()
{
  // init variables
  System.KeyboardMap= 0LL;
  System.BufRead= System.BufWrite= 0;
  System.LastKeyPress= -1;
  System.ONPlusTask= 0;
  System.RS232Speed= 0;
}

unsigned long long KeyCodeToKeyMap(int key)
{
  if ((key<11)||(key>75)||((key%10)<1)||((key%10)>6)) return 0LL;
  key-= 11;
  key= (key/10)*6+(key%10);
  return 1LL<<key;
}

